%------------Calculating thrust in irregular waves-------------------------
%-----------------------by foil propulsion---------------------------------
%------------------Written by Eivind Finne Riley---------------------------
%---------------------Part of Master Thesis-------------------------------
%----------------NTNU, Dept. of Marine Technology--------------------------
%---------------------------June 2015----------------------------------

%--------Number of input frequencies must be 99 w/o modifying script------
%-------------------Input directory must be defined-----------------------
clc
clear all
close all
                                                                                                            
%-------------------------Wave/vessel/ocean/foil data input----------------

Hs=5;                      %Change    
Tp     = 7;             %Change
beta = 0;                 %Change  (heading) [rad]
rho =1025;                    
g   =9.81;
fp  =1./Tp;
alphajonswap = 0.036 - 0.0056*(Tp/sqrt(Hs));
gamma   = exp(3.484*(1-0.1975*alphajonswap*(Tp^4)/(Hs^2)));
sigma_a = 0.07;
sigma_b = 0.09;
delta_f = 0.1/(2*pi);

U = 6*0.51444;

c = 2;                   %Chord length
s = 12;                     %Span
aspect = s/c;              
xc=38.5;                      %Position relative to longitudinal pivot point
zc = -4;                 %Submergence depth of the foils

yPORT = -s/2;
ySTAR = s/2;

%------Theodorsen coefficients
A1 = 0.165;
A2 = 0.335;
b1 = 0.0455;
b2 = 0.3;

%Theodorsen
W=2*pi/Tp;
W_E = W + (W^2/g)*cos(beta);
KR=W_E*c/(2*U);
C_KR =  1 - (i*KR*A1)/(b1+i*KR) - (i*KR*A2)/(b2+i*KR);
C_KR = abs(C_KR);


%--------------Calculates the JONSWAP specter------------------------------

w=0.22319:0.013751:pi/2; %Frequencies matched to ShipX input
f=w./(2*pi);

for j = 1:length(Tp)
    for k  = 1:length (Hs)
        Sf = zeros (1,size(f,2));
        Sw = zeros(1,size(f,2));
        alpha(j,k)=(Hs(1,k)/(4*g))^2*(2*pi*fp(1,j))^4*(0.065*gamma^0.803...
            + 0.135)^-1;
        for i = 1:length(f)
            if f(1,i)<= fp(1,j)
                sigma = sigma_a;
            else 
                sigma = sigma_b;
            end
            Sf(1,i)=alpha(j,k)*g^2*(2*pi)^-4*f(1,i)^-5*exp(-5/4*(fp(1,j...
                )/f(1,i))^4)*gamma^exp(-(f(1,i)/fp(1,j)-1)^2/(2*sigma^2));
            Sw(1,i)=Sf(1,i)/(2*pi);
        end

            
        end
end

%------Imports Real parts of RAOs (MATLAB generated script)-----------------------


filename = 'C:\Matlab\input.re1';

if beta==0

    startRow = 14;                                                              %Changes for each heading. Note that this interval is for numw = 99
    endRow = 705;                                                               %Changes for each heading. Note that this interval is for numw = 99
elseif beta == pi/8
    startRow = 708;
    endRow = 1399;
elseif beta == pi/4
    startRow = 1402;
    endRow = 2093;
elseif beta == 3*pi/8
    startRow=2096;
    endRow=2787;
elseif beta == pi/2
    startRow = 2790;
    endRow = 3481;
elseif beta == 5*pi/8
    startRow = 3484;
    endRow = 4175;
elseif beta == 3*pi/4
    startRow = 4178;
    endRow = 4869;
elseif beta == 7*pi/8
    startRow = 4872;
    endRow = 5563;
elseif beta == pi
    startRow = 5566;
    endRow = 6257;
else
    disp('Invalid value for beta')
    return
end

    
    
    
    formatSpec = '%*3s%16s%[^\n\r]';

fileID = fopen(filename,'r');

dataArray = textscan(fileID, formatSpec, endRow-startRow+1, 'Delimiter', '', 'WhiteSpace', '', 'HeaderLines', startRow-1, 'ReturnOnError', false);

fclose(fileID);

raw = repmat({''},length(dataArray{1}),length(dataArray)-1);
for col=1:length(dataArray)-1
    raw(1:length(dataArray{col}),col) = dataArray{col};
end
numericData = NaN(size(dataArray{1},1),size(dataArray,2));


rawData = dataArray{1};
for row=1:size(rawData, 1);

    regexstr = '(?<prefix>.*?)(?<numbers>([-]*(\d+[\,]*)+[\.]{0,1}\d*[eEdD]{0,1}[-+]*\d*[i]{0,1})|([-]*(\d+[\,]*)*[\.]{1,1}\d+[eEdD]{0,1}[-+]*\d*[i]{0,1}))(?<suffix>.*)';
    try
        result = regexp(rawData{row}, regexstr, 'names');
        numbers = result.numbers;

        invalidThousandsSeparator = false;
        if any(numbers==',');
            thousandsRegExp = '^\d+?(\,\d{3})*\.{0,1}\d*$';
            if isempty(regexp(thousandsRegExp, ',', 'once'));
                numbers = NaN;
                invalidThousandsSeparator = true;
            end
        end
        if ~invalidThousandsSeparator;
            numbers = textscan(strrep(numbers, ',', ''), '%f');
            numericData(row, 1) = numbers{1};
            raw{row, 1} = numbers{1};
        end
    catch me
    end
end

input = cell2mat(raw);
clearvars filename startRow endRow formatSpec fileID dataArray ans raw col numericData rawData row regexstr result numbers invalidThousandsSeparator thousandsRegExp me;

reRAO = input;

for i = 0:98
    reRAO_Heave(i+1)=reRAO(3 + 7*i);
    reRAO_Roll(i+1) = reRAO(4 + 7*i);
    reRAO_Pitch(i+1) = reRAO(5+ 7*i);
end


%--------------------Imports imaginary RAOs--------------------------------


filename = 'C:\Matlab\input.re1';


if beta==0

    startRow = 14;                                                              %Changes for each heading. Note that this interval is for numw = 99
    endRow = 705;                                                               %Changes for each heading. Note that this interval is for numw = 99
elseif beta == pi/8
    startRow = 708;
    endRow = 1399;
elseif beta == pi/4
    startRow = 1402;
    endRow = 2093;
elseif beta == 3*pi/8
    startRow=2096;
    endRow=2787;
elseif beta == pi/2
    startRow = 2790;
    endRow = 3481;
elseif beta == 5*pi/8
    startRow = 3484;
    endRow = 4175;
elseif beta == 3*pi/4
    startRow = 4178;
    endRow = 4869;
elseif beta == 7*pi/8
    startRow = 4872;
    endRow = 5563;
elseif beta == pi
    startRow = 5566;
    endRow = 6257;
else
    disp('Invalid value for beta')
    return
end

formatSpec = '%*19s%s%[^\n\r]';
fileID = fopen(filename,'r');
dataArray = textscan(fileID, formatSpec, endRow-startRow+1, 'Delimiter', '', 'WhiteSpace', '', 'HeaderLines', startRow-1, 'ReturnOnError', false);
fclose(fileID);
raw = repmat({''},length(dataArray{1}),length(dataArray)-1);
for col=1:length(dataArray)-1
    raw(1:length(dataArray{col}),col) = dataArray{col};
end
numericData = NaN(size(dataArray{1},1),size(dataArray,2));

rawData = dataArray{1};
for row=1:size(rawData, 1);
    regexstr = '(?<prefix>.*?)(?<numbers>([-]*(\d+[\,]*)+[\.]{0,1}\d*[eEdD]{0,1}[-+]*\d*[i]{0,1})|([-]*(\d+[\,]*)*[\.]{1,1}\d+[eEdD]{0,1}[-+]*\d*[i]{0,1}))(?<suffix>.*)';
    try
        result = regexp(rawData{row}, regexstr, 'names');
        numbers = result.numbers;
        invalidThousandsSeparator = false;
        if any(numbers==',');
            thousandsRegExp = '^\d+?(\,\d{3})*\.{0,1}\d*$';
            if isempty(regexp(thousandsRegExp, ',', 'once'));
                numbers = NaN;
                invalidThousandsSeparator = true;
            end
        end
        if ~invalidThousandsSeparator;
            numbers = textscan(strrep(numbers, ',', ''), '%f');
            numericData(row, 1) = numbers{1};
            raw{row, 1} = numbers{1};
        end
    catch me
    end
end


R = cellfun(@(x) ~isnumeric(x) && ~islogical(x),raw); 
raw(R) = {NaN}; 

input = cell2mat(raw);

clearvars filename startRow endRow formatSpec fileID dataArray ans raw col numericData rawData row regexstr result numbers invalidThousandsSeparator thousandsRegExp me R;

imRAO = input;

for i = 0:98
    imRAO_Heave(i+1)=imRAO(3 + 7*i);
    imRAO_Roll(i+1) = imRAO(4 + 7*i);
    imRAO_Pitch(i+1) = imRAO(5+ 7*i);
end


%---------Defines the RAOs at midpoint DW values---------------------------
for i=1:98
    imRAO_Heave(i) = (imRAO_Heave(i) + imRAO_Heave(i+1))/2;
    imRAO_Roll(i) = (imRAO_Roll(i) + imRAO_Roll(i+1))/2;
    imRAO_Pitch(i) = (imRAO_Pitch(i) + imRAO_Pitch(i+1))/2;
    reRAO_Heave(i) = (reRAO_Heave(i) + reRAO_Heave(i+1))/2;
    reRAO_Roll(i) = (reRAO_Roll(i) + reRAO_Roll(i+1))/2;
    reRAO_Pitch(i) = (reRAO_Pitch(i) + reRAO_Pitch(i+1))/2;
end

i=sqrt(-1);
%--------------------------------------------------------------------------

Time = 120;                                         % [s]
dt = 0.5;                                            % [s]
numT = Time/dt;

nfreq = 99; %Has to match ShipX input: number of frequencies
DW = zeros(nfreq-1,1);

%Defines random wave phases
for j = 1:nfreq-1
    epsilon(j)=rand()*2*pi;
end

%If you want to keep phases to reproduce results, keep reproduce = 1
reproduce=1;
if reproduce==1;
epsilon=[5.1191
    5.6913
    0.7979
    5.7389
    3.9732
    0.6129
    1.7499
    3.4362
    6.0162
    6.0626
    0.9903
    6.0984
    6.0141
    3.0497
    5.0283
    0.8915
    2.6500
    5.7537
    4.9776
    6.0287
    4.1201
    0.2244
    5.3352
    5.8685
    4.2646
    4.7610
    4.6692
    2.4644
    4.1185
    1.0756
    4.4362
    0.2000
    1.7400
    0.2901
    0.6103
    5.1739
    4.3657
    1.9924
    5.9704
    0.2164
    2.7567
    2.3974
    4.8099
    4.9964
    1.1742
    3.0773
    2.7997
    4.0609
    4.4571
    4.7418
    1.7343
    4.2707
    4.1161
    1.0217
    0.7477
    3.1313
    6.0302
    2.1387
    3.6773
    1.4063
    4.7204
    1.6028
    3.1790
    4.3924
    5.5977
    6.0274
    3.4383
    0.8710
    0.9380
    1.6180
    5.2824
    1.5977
    5.1163
    1.5301
    5.8387
    2.1990
    1.2352
    1.5776
    3.8707
    2.9738
    2.2095
    5.2203
    3.6773
    3.4540
    5.7629
    1.7960
    4.7576
    4.7358
    2.3904
    3.5677
    0.4766
    0.3390
    3.3351
    4.8957
    5.8686
    0.8162
    3.5740
    2.9493]';    
end


%Initiates time series defining ship&foil displacements, velocities and
%accellerations, and wave particle velocities and accelleration
for t=1:numT
timeseries(t) = -dt + t*dt;
    for j=1:nfreq-1
        if t==1
                DW(j) = w(j+1)-w(j);                                               %freq interval is slightly different for each freq step due to rounding in shipX
                Amp1W(j)=sqrt(2*Sw(1,j)*DW(j));                                     %Wave elevation amp for each freq. band
                Amp2W(j)=sqrt(2*Sw(1,j+1)*DW(j));
                AmpW(j) = (Amp1W(j)+Amp2W(j))/2;
                w(j) = w(j) + (w(j+1) - w(j))/2;                                   %Sets the frequencies at midpoint DW values
                w_e(j) =w(j) + (w(j)^2)./g.*U*cos(beta);                           %Freq. of encounter
                k(j) = (w(j)^2)/g;                                                 %Wave number
        end %endif
        
%DISPLACEMENTS
        %Determine foil position due to linear wave theory for each freq.component
        %AT COG:
        Hdisp(j) = (1/w_e(j))*AmpW(j)*abs(reRAO_Heave(j))*cos(w_e(j)*dt*t + atan(imRAO_Heave(j)/reRAO_Heave(j))+epsilon(j));
        Pdisp(j) = (1/w_e(j))*AmpW(j)*abs(reRAO_Pitch(j))*cos(w_e(j)*dt*t + atan(imRAO_Pitch(j)/reRAO_Pitch(j))+epsilon(j));   
        %At foil position: 
        eta3foilfreq(j)= Hdisp(j) - xc*Pdisp(j); %Vertical foil position for each freq comp
        
%VELOCITIES
        %Heave induced velocity:
        Hvel(j) = -AmpW(j)*abs(reRAO_Heave(j))*sin(w_e(j)*dt*t + atan(imRAO_Heave(j)/reRAO_Heave(j)) + epsilon(j));
        %Pitch velocity induced motions:
        Pvel(j) = -AmpW(j)*abs(reRAO_Pitch(j))*sin(w_e(j)*dt*t + atan(imRAO_Pitch(j)/reRAO_Pitch(j)) + epsilon(j));
        foilheavefrompitchmotion(j) = -Pvel(j)*xc;
        %Pitch displacement induced motions: 
        FoilVerticalFlowFromPitchDisp(j)=U*Pdisp(j);
        
%ACCELLERATIONS
        %Heave accelleration of ship
        H_acc(j) = -w_e(j)*AmpW(j)*abs(reRAO_Heave(j))*sin(w_e(j)*dt*t + atan(imRAO_Heave(j)/reRAO_Heave(j)) + epsilon(j)); 
        %Heave accelleration of foil due to ship pitch accelleration
        P_acc(j) = -w_e(j)*AmpW(j)*abs(reRAO_Pitch(j))*sin(w_e(j)*dt*t + atan(imRAO_Heave(j)/reRAO_Heave(j)) + epsilon(j))*xc;
       
        
    end %end freq
    
%DISPLACEMENTS
    eta3foil(t)= zc + sum(eta3foilfreq); %Finds vertical foil position summed from all freq. components for each time step
    Pdisptot(t) = sum(Pdisp);
%VELOCITIES
    HeavevelocityFOIL(t) = sum(Hvel); %Sums up heave velocity for all freq comps
    HeaveFOILfrompitchmotion(t) = sum(foilheavefrompitchmotion); %Sums up heave velocity from pitch velocity for all freq comps
    VerticalFlowPitchDispl(t) = sum(FoilVerticalFlowFromPitchDisp); %Sums up heave velocity from pitch displacement for all freq comps
    P_VEL(t)=sum(Pvel);
%ACCELLERATION
    H_ACC(t) = sum(H_acc);
    P_ACC(t) = sum(P_acc);
end %end time



%Time series for calculating fluid particle velocities and accellerations
%at foil position

for t=1:numT
    for j=1:nfreq-1
        
%WAVE PARTICLE VELOCITIES
        wv(j)  = w(j)*AmpW(j)*exp(k(j)*eta3foil(t))*cos(w_e(j)*dt*t - k(j)*xc*cos(beta) + epsilon(j)); 
        uv(j) = -w(j)*cos(beta)*AmpW(j)*cos(beta)*exp(k(j)*eta3foil(t))*sin(w_e(j)*dt*t - k(j)*xc*cos(beta)  +epsilon(j));
%WAVE PARTICLE ACCELLERATIONS
        a_w(j) = w(j)*w_e(j)*AmpW(j)*exp(k(j)*zc)*cos(w_e(j)*dt*t - k(j)*xc*cos(beta)); %vertical accelleration   
    end %end freq
%WAVE PARTICLE VELOCITY
    Wv(t) = sum(wv);
    Uv(t) = sum(uv);
%WAVE PARTICLE ACCELLERATION
    Aw(t) = sum(a_w);
end %end time

%Calculates relative velocities
wrel = Wv' - ( HeavevelocityFOIL' - HeaveFOILfrompitchmotion' - VerticalFlowPitchDispl');  %vertical wave particle velocity
urel = U + Uv; %horizontal wave particle velocity

%Time series for instantaneous angle of attack  
for t=1:numT
    alpha(t) = atan(wrel(t)/urel(t)) - Pdisptot(t);
    alpha(t) = alpha(t)*C_KR;
end

for t=1:numT %Time series for calculating alphazero, used to find inflow velocity
        alphazero(t) = alpha(t)-Pdisptot(t);
end

for t=1:numT %Inflow velocity time series
        Vin(t) = (urel(t))/cos(alphazero(t));
end



%---------------------LIFT-------------------------------------------------
%Instantaneous lift coefficient: Input for Gaussian fit w/7terms below
       a1 =      0.8952;
       b1 =      0.2448;
       c1 =      0.1237;
       a2 =      0.3728;
       b2 =      0.4081;
       c2 =     0.08667;
       a3 =       0.133;
       b3 =      0.4718;
       c3 =     0.03185;
       a4 =      0.2469;
       b4 =      0.1083;
       c4 =      0.0762;
       a5 =           0;
       b5 =      0.7441;
       c5 =    0.003392;
       a6 =      0.5628;%  (-0.0629, 1.189)
       b6 =      0.6323;%  (0.5195, 0.745)
       c6 =      0.2932;%  (0.07691, 0.5094)
       a7 =       0.596;%  (0.2086, 0.9834)
       b7 =       1.044;%  (0.7523, 1.335)
       c7 =      0.3908;%  (0.2193, 0.5623)

     
 
       %Time series for lift coefficient, NACA0015 corrected for finite span          
 for t=1:numT
     liftcoeff(t)  = a1*exp(-((abs(alpha(t))-b1)/c1)^2) + a2*exp(-((abs(alpha(t))-b2)/c2)^2) + a3*exp(-((abs(alpha(t))-b3)/c3)^2) + a4*exp(-((abs(alpha(t))-b4)/c4)^2) + a5*exp(-((abs(alpha(t))-b5)/c5)^2) + a6*exp(-((abs(alpha(t))-b6)/c6)^2) + a7*exp(-((abs(alpha(t))-b7)/c7)^2);
 end
 
%Added mass forces 
 AC = rho*pi*(c/2)^2*s;

 for t=1:numT
 RelAccVert(t) = Aw(t) - (H_ACC(t) - P_ACC(t) - P_VEL(t)*urel(t)); %Vertical relative acceleration at foil position
 F_added(t) = RelAccVert(t)*AC;
 F_added(t) = abs(F_added(t));
 end
 
 Kf=0.8*10^6;
 
 
 for t=1:numT
 L(t) = 0.5*rho*c*s*Vin(t)^2*liftcoeff(t);    
 end
 
 alpha_init = alpha;
 
 for t=1:numT
   Mk(t)=0;
   alpha(t)=abs(alpha(t));
   count=0;
   
   L(t) = 0.5*rho*c*s*Vin(t)^2*liftcoeff(t);
   M(t) = L(t)*c/8*cos(alphazero(t) - Pdisptot(t)) + F_added(t)*3*c/8*cos(Pdisptot(t));
   M_init(t) = M(t);
   Diff(t) = M(t) - Mk(t);
   t
        while abs(Diff(t)) > 1000
        count=count+1;    
        
       if alpha(t) > 0 
            deltafoilpitch = -0.001;
            alpha(t)=alpha(t)+deltafoilpitch;
        elseif alpha(t) < 0
            deltafoilpitch = 0.001;
            alpha(t) = alpha(t)+deltafoilpitch;
            
        end
        liftcoeff(t) = a1*exp(-((abs(alpha(t))-b1)/c1)^2) + a2*exp(-((abs(alpha(t))-b2)/c2)^2) + a3*exp(-((abs(alpha(t))-b3)/c3)^2) + a4*exp(-((abs(alpha(t))-b4)/c4)^2) + a5*exp(-((abs(alpha(t))-b5)/c5)^2) + a6*exp(-((abs(alpha(t))-b6)/c6)^2) + a7*exp(-((abs(alpha(t))-b7)/c7)^2);
        L(t) = 0.5*rho*c*s*Vin(t)^2*liftcoeff(t);
        M(t) = L(t)*c/8*cos(alphazero(t) - Pdisptot(t)) + F_added(t)*3*c/8*cos(Pdisptot(t));
        Mk(t) = Kf*count*abs(deltafoilpitch);
   
        Diff(t)= M(t) - Mk(t);
   end
    
 end
 


%----------------------------DRAG------------------------------------------
%Instantaneous drag coefficient: Input for Gaussian fit w/7 terms below

       a1 =      0.4145;
       b1 =        1.61;
       c1 =      0.1331;
       a2 =      0.8715;
       b2 =      0.8973;
       c2 =      0.3023;
       a3 =           0;
       b3 =        2.87;
       c3 =     0.04941;
       a4 =        1.51;
       b4 =      0.5719;
       c4 =      0.1561;
       a5 =           0;
       b5 =       1.055;
       c5 =    2.22e-14;
       a6 =       1.687;%  (-25.93, 29.31)
       b6 =         1.4;%  (-4.124, 6.924)
       c6 =      0.4044;%  (-21.75, 22.56)
       a7 =      -1.173;%  (-65, 62.65)
       b7 =      0.5746;%  (0.4617, 0.6875)
       c7 =      0.1314;%  (-0.4846, 0.7474)

       
       
%Time series for drag coeff NACA       
for t=1:numT
    
    dragcoeff(t)  = a1*exp(-((abs(alpha(t))-b1)/c1)^2) + a2*exp(-((abs(alpha(t))-b2)/c2)^2) + a3*exp(-((abs(alpha(t))-b3)/c3)^2) + a4*exp(-((abs(alpha(t))-b4)/c4)^2) + a5*exp(-((abs(alpha(t))-b5)/c5)^2) + a6*exp(-((abs(alpha(t))-b6)/c6)^2) + a7*exp(-((abs(alpha(t))-b7)/c7)^2); 
    D(t) = 0.5*rho*c*s*Vin(t)^2*dragcoeff(t);
    
end %end time

for t=1:numT
   T(t) = L(t)*sin(abs(alphazero(t))) - D(t)*cos(abs(alphazero(t))) + 2014.7; %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end

%Viscous strut resistance
strutC=0.8;
strutT=0.3;
v = 1.18*10^-6;
RNSTRUT = U*strutC/v;
CFSTRUT = 0.075/((log10(RNSTRUT-2))^2);
CDSTRUT = 2*(1+2*strutT/strutC)*CFSTRUT;
Visc_strut_res = 0.5*rho*U^2*CDSTRUT*abs(zc)*2;

%Strut spray resistance
spray_res = 2* 0.24*0.5*rho*U^2*strutT^2;

%Viscous foil drag
Viscdragcoeff=0.015;
Visc_drag = 0.5*rho*U^2*c*s*Viscdragcoeff;T= T - Visc_drag - Visc_strut_res;

Thrust = T - Visc_drag - Visc_strut_res - spray_res;

HsTp = [Hs, Tp]

Meanthrust = mean(Thrust);

Meanthrust = Meanthrust/1000 %converts to kN



%----------------------------------PLOTS------------------------------------------------

    %figure                             %Vertical Position of foil
    %plot(timeseries, eta3foil)
    %title('Vertical position of foil [m] relative to calm water line, time series')

    figure
    plot(timeseries, alpha*180/pi)
    hold on
    plot(timeseries, abs(alpha_init*180/pi))    
























