%------------Calculating thrust in irregular waves-------------------------
%-----------------------by foil propulsion---------------------------------
%------------------Written by Eivind Finne Riley---------------------------
%---------------------Part of Master Thesis-------------------------------
%----------------NTNU, Dept. of Marine Technology--------------------------
%---------------------------June 2015----------------------------------

%--------Number of input frequencies must be 99 w/o modifying script------
%-------------------Input directory must be defined-----------------------
clc
clear all
close all
                                                                                                            
%-------------------------Wave/vessel/ocean/foil data input----------------

Hs=3;                      %Change    
Tp     = 6.5;             %Change
beta = 0;                 %Change  (heading) [rad]
rho =1025;                    
g   =9.81;
fp  =1./Tp;
alphajonswap = 0.036 - 0.0056*(Tp/sqrt(Hs));
gamma   = exp(3.484*(1-0.1975*alphajonswap*(Tp^4)/(Hs^2)));
sigma_a = 0.07;
sigma_b = 0.09;
delta_f = 0.1/(2*pi);

U = 6*0.51444;

c = 2;                   %Chord length
s = 12;                     %Span
aspect = s/c;              
xc=38.5;                      %Position relative to longitudinal pivot point
zc = -4;                 %Submergence depth of the foils

yPORT = -s/2;
ySTAR = s/2;

%------Theodorsen coefficients
A1 = 0.165;
A2 = 0.335;
b1 = 0.0455;
b2 = 0.3;

%------The below defines y-positions of foil strips. No. of strips are 20

nstrips=20;

for i=1:nstrips
       if i == 1
           ypos(i) = yPORT + ((ySTAR-yPORT)/(2*nstrips))*i;
       else
           ypos(i) = ypos(i-1) + (ySTAR-yPORT)/nstrips;
       end
end

ds = s/nstrips;

%Lifting line coefficients
a = (pi*rho*c*s/nstrips*U^2)/(1+2/aspect);
b = (2*pi*aspect*rho*c*s/nstrips*U^2)/(aspect+2)^2;    



%--------------Calculates the JONSWAP specter------------------------------

w=0.22319:0.013751:pi/2; %Frequencies matched to ShipX input
f=w./(2*pi);

for j = 1:length(Tp)
    for k  = 1:length (Hs)
        Sf = zeros (1,size(f,2));
        Sw = zeros(1,size(f,2));
        alpha(j,k)=(Hs(1,k)/(4*g))^2*(2*pi*fp(1,j))^4*(0.065*gamma^0.803...
            + 0.135)^-1;
        for i = 1:length(f)
            if f(1,i)<= fp(1,j)
                sigma = sigma_a;
            else 
                sigma = sigma_b;
            end
            Sf(1,i)=alpha(j,k)*g^2*(2*pi)^-4*f(1,i)^-5*exp(-5/4*(fp(1,j...
                )/f(1,i))^4)*gamma^exp(-(f(1,i)/fp(1,j)-1)^2/(2*sigma^2));
            Sw(1,i)=Sf(1,i)/(2*pi);
        end

            
        end
end

%------Imports Real parts of RAOs (MATLAB generated script)-----------------------


filename = 'C:\Matlab\input.re1';

if beta==0

    startRow = 14;                                                              %Changes for each heading. Note that this interval is for numw = 99
    endRow = 705;                                                               %Changes for each heading. Note that this interval is for numw = 99
elseif beta == pi/8
    startRow = 708;
    endRow = 1399;
elseif beta == pi/4
    startRow = 1402;
    endRow = 2093;
elseif beta == 3*pi/8
    startRow=2096;
    endRow=2787;
elseif beta == pi/2
    startRow = 2790;
    endRow = 3481;
elseif beta == 5*pi/8
    startRow = 3484;
    endRow = 4175;
elseif beta == 3*pi/4
    startRow = 4178;
    endRow = 4869;
elseif beta == 7*pi/8
    startRow = 4872;
    endRow = 5563;
elseif beta == pi
    startRow = 5566;
    endRow = 6257;
else
    disp('Invalid value for beta')
    return
end

    
    
    
    formatSpec = '%*3s%16s%[^\n\r]';

fileID = fopen(filename,'r');

dataArray = textscan(fileID, formatSpec, endRow-startRow+1, 'Delimiter', '', 'WhiteSpace', '', 'HeaderLines', startRow-1, 'ReturnOnError', false);

fclose(fileID);

raw = repmat({''},length(dataArray{1}),length(dataArray)-1);
for col=1:length(dataArray)-1
    raw(1:length(dataArray{col}),col) = dataArray{col};
end
numericData = NaN(size(dataArray{1},1),size(dataArray,2));


rawData = dataArray{1};
for row=1:size(rawData, 1);

    regexstr = '(?<prefix>.*?)(?<numbers>([-]*(\d+[\,]*)+[\.]{0,1}\d*[eEdD]{0,1}[-+]*\d*[i]{0,1})|([-]*(\d+[\,]*)*[\.]{1,1}\d+[eEdD]{0,1}[-+]*\d*[i]{0,1}))(?<suffix>.*)';
    try
        result = regexp(rawData{row}, regexstr, 'names');
        numbers = result.numbers;

        invalidThousandsSeparator = false;
        if any(numbers==',');
            thousandsRegExp = '^\d+?(\,\d{3})*\.{0,1}\d*$';
            if isempty(regexp(thousandsRegExp, ',', 'once'));
                numbers = NaN;
                invalidThousandsSeparator = true;
            end
        end
        if ~invalidThousandsSeparator;
            numbers = textscan(strrep(numbers, ',', ''), '%f');
            numericData(row, 1) = numbers{1};
            raw{row, 1} = numbers{1};
        end
    catch me
    end
end

input = cell2mat(raw);
clearvars filename startRow endRow formatSpec fileID dataArray ans raw col numericData rawData row regexstr result numbers invalidThousandsSeparator thousandsRegExp me;

reRAO = input;

for i = 0:98
    reRAO_Heave(i+1)=reRAO(3 + 7*i);
    reRAO_Roll(i+1) = reRAO(4 + 7*i);
    reRAO_Pitch(i+1) = reRAO(5+ 7*i);
end


%--------------------Imports imaginary RAOs--------------------------------


filename = 'C:\Matlab\input.re1';


if beta==0

    startRow = 14;                                                              %Changes for each heading. Note that this interval is for numw = 99
    endRow = 705;                                                               %Changes for each heading. Note that this interval is for numw = 99
elseif beta == pi/8
    startRow = 708;
    endRow = 1399;
elseif beta == pi/4
    startRow = 1402;
    endRow = 2093;
elseif beta == 3*pi/8
    startRow=2096;
    endRow=2787;
elseif beta == pi/2
    startRow = 2790;
    endRow = 3481;
elseif beta == 5*pi/8
    startRow = 3484;
    endRow = 4175;
elseif beta == 3*pi/4
    startRow = 4178;
    endRow = 4869;
elseif beta == 7*pi/8
    startRow = 4872;
    endRow = 5563;
elseif beta == pi
    startRow = 5566;
    endRow = 6257;
else
    disp('Invalid value for beta')
    return
end

formatSpec = '%*19s%s%[^\n\r]';
fileID = fopen(filename,'r');
dataArray = textscan(fileID, formatSpec, endRow-startRow+1, 'Delimiter', '', 'WhiteSpace', '', 'HeaderLines', startRow-1, 'ReturnOnError', false);
fclose(fileID);
raw = repmat({''},length(dataArray{1}),length(dataArray)-1);
for col=1:length(dataArray)-1
    raw(1:length(dataArray{col}),col) = dataArray{col};
end
numericData = NaN(size(dataArray{1},1),size(dataArray,2));

rawData = dataArray{1};
for row=1:size(rawData, 1);
    regexstr = '(?<prefix>.*?)(?<numbers>([-]*(\d+[\,]*)+[\.]{0,1}\d*[eEdD]{0,1}[-+]*\d*[i]{0,1})|([-]*(\d+[\,]*)*[\.]{1,1}\d+[eEdD]{0,1}[-+]*\d*[i]{0,1}))(?<suffix>.*)';
    try
        result = regexp(rawData{row}, regexstr, 'names');
        numbers = result.numbers;
        invalidThousandsSeparator = false;
        if any(numbers==',');
            thousandsRegExp = '^\d+?(\,\d{3})*\.{0,1}\d*$';
            if isempty(regexp(thousandsRegExp, ',', 'once'));
                numbers = NaN;
                invalidThousandsSeparator = true;
            end
        end
        if ~invalidThousandsSeparator;
            numbers = textscan(strrep(numbers, ',', ''), '%f');
            numericData(row, 1) = numbers{1};
            raw{row, 1} = numbers{1};
        end
    catch me
    end
end


R = cellfun(@(x) ~isnumeric(x) && ~islogical(x),raw); 
raw(R) = {NaN}; 

input = cell2mat(raw);

clearvars filename startRow endRow formatSpec fileID dataArray ans raw col numericData rawData row regexstr result numbers invalidThousandsSeparator thousandsRegExp me R;

imRAO = input;

for i = 0:98
    imRAO_Heave(i+1)=imRAO(3 + 7*i);
    imRAO_Roll(i+1) = imRAO(4 + 7*i);
    imRAO_Pitch(i+1) = imRAO(5+ 7*i);
end


%---------Defines the RAOs at midpoint DW values---------------------------
for i=1:98
    imRAO_Heave(i) = (imRAO_Heave(i) + imRAO_Heave(i+1))/2;
    imRAO_Roll(i) = (imRAO_Roll(i) + imRAO_Roll(i+1))/2;
    imRAO_Pitch(i) = (imRAO_Pitch(i) + imRAO_Pitch(i+1))/2;
    reRAO_Heave(i) = (reRAO_Heave(i) + reRAO_Heave(i+1))/2;
    reRAO_Roll(i) = (reRAO_Roll(i) + reRAO_Roll(i+1))/2;
    reRAO_Pitch(i) = (reRAO_Pitch(i) + reRAO_Pitch(i+1))/2;
end

i=sqrt(-1);
%--------------------------------------------------------------------------

Time = 12;                                         % [s]
dt = 0.5;                                            % [s]
numT = Time/dt;

nfreq = 99; %Has to match ShipX input: number of frequencies
DW = zeros(nfreq-1,1);


%----------------Begins thrust calculation---------------------------------

%Initiates time series to calculate vertical foil position as function of
%time:

for j = 1:nfreq-1
    epsilon(j)=rand()*2*pi;
end

for t=1:numT
timeseries(t) = -dt + t*dt;

    for j=1:nfreq-1
    
        if t==1
         DW(j) = w(j+1)-w(j);                                               %freq interval is slightly different for each freq step due to rounding in shipX
         Amp1W(j)=sqrt(2*Sw(1,j)*DW(j));                                     %Wave elevation amp for each freq. band
         Amp2W(j)=sqrt(2*Sw(1,j+1)*DW(j));
         AmpW(j) = (Amp1W(j)+Amp2W(j))/2;
         w(j) = w(j) + (w(j+1) - w(j))/2;                                   %Sets the frequencies at midpoint DW values
         w_e(j) =w(j) + (w(j)^2)./g.*U*cos(beta);                           %Freq. of encounter
         k(j) = (w(j)^2)/g;                                                 %Wave number
        end
        
    %Determine foil position due to linear wave theory for each freq.
    %component
 
    %AT COG:
        Hdisp(j) = (1/w_e(j))*AmpW(j)*abs(reRAO_Heave(j))*cos(w_e(j)*dt*t + atan(imRAO_Heave(j)/reRAO_Heave(j))+epsilon(j));
        Rdisp(j) = (1/w_e(j))*AmpW(j)*abs(reRAO_Roll(j))*cos(w_e(j)*dt*t + atan(imRAO_Roll(j)/reRAO_Roll(j))+epsilon(j));
        Pdisp(j) = (1/w_e(j))*AmpW(j)*abs(reRAO_Pitch(j))*cos(w_e(j)*dt*t + atan(imRAO_Pitch(j)/reRAO_Pitch(j))+epsilon(j));
 
        
            for m=1:length(ypos) %Calculates vertical position of each foil strip, for each freq
     
                eta3FOILfreq(j,m) = Hdisp(j) + ypos(m)*Rdisp(j) - xc*Pdisp(j); %Vertical foil position from each frequency
    
            end %end foil strip loop
    end          %end freq
    
    Pdisptot(t) = sum(Pdisp); %Sums up pitch displacement from all freq comps, for use in angle of attack
    
    for m=1:length(ypos) 
        eta3FOIL(t,m) = zc + sum(eta3FOILfreq(:,m)); %Sums all vertical displacements from all frequencues, for all strips
    end %end strip
    
end  %end timestep

disp('Heave position calculation.....................................done')
%Initiates time series to calculate wave motion at foil position
for t=1:numT
 
    for j = nfreq-1
        
        for m = 1:length(ypos)
        wv(j,m)  = w(j)*AmpW(j)*exp(k(j)*eta3FOIL(t))*cos(w_e(j)*dt*t - k(j)*xc*cos(beta) - k(j)*ypos(m)*sin(beta) + epsilon(j)); 
        uv(j,m) = -w(j)*cos(beta)*AmpW(j)*cos(beta)*exp(k(j)*eta3FOIL(t))*sin(w_e(j)*dt*t - k(j)*xc*cos(beta) - k(j)*ypos(m)*sin(beta) +epsilon(j));
        end %end strip
    end %End freq
    
    for m=1:length(ypos)
        wavevel_w(t,m) = sum(wv(:,m)); %Sums vertical wave induced velocities at foil strip position for all freq. components
        wavevel_v(t,m) = sum(uv(:,m)); %Sums horizontal wave induced velocities at foil strip position for all freq. components
    end %end strip
end %End time

disp('Wave motion calculation........................................done')
    


for t = 1:numT    
   
    for j = 1:nfreq-1
        
        %Heave induced velocity:
        Hvel(j) = -AmpW(j)*abs(reRAO_Heave(j))*sin(w_e(j)*dt*t + atan(imRAO_Heave(j)/reRAO_Heave(j)) + epsilon(j));
        
        %Pitch velocity induced motions:
        Pvel(j) = -AmpW(j)*abs(reRAO_Pitch(j))*sin(w_e(j)*dt*t + atan(imRAO_Pitch(j)/reRAO_Pitch(j)) + epsilon(j));
        foilheavefrompitchmotion(j) = -Pvel(j)*xc;
        
        %Pitch displacement induced motions:
        Pdisp(j) = (1/w_e(j))*AmpW(j)*abs(reRAO_Pitch(j))*cos(w_e(j)*dt*t + atan(imRAO_Pitch(j)/reRAO_Pitch(j))+ epsilon(j));  
        FoilVerticalFlowFromPitchDisp(j)=U*Pdisp(j);
        
        %Roll velocity induced motions
        Rvel(j) = -AmpW(j)*abs(reRAO_Roll(j))*sin(w_e(j)*dt*t + atan(imRAO_Pitch(j)/reRAO_Pitch(j)) + epsilon(j));
        
        
    end %end freq
    
        Rveltotal(t)=sum(Rvel); %sums roll velocity for all freq components
   
    for m=1:length(ypos)
            
        HeavevelocityFOIL(t,m) = sum(Hvel);   %Sums up heave velocity for all freq comps
        HeaveFOILfrompitchmotion(t,m) = sum(foilheavefrompitchmotion); %Sums up heave velocity from pitch velocity for all freq comps
        VerticalFlowPitchDispl(t,m) = sum(FoilVerticalFlowFromPitchDisp); %Sums up heave velocity from pitch displacement for all freq comps
        
        VerticalFlowRollMotion(t,m) = Rveltotal(t)*ypos(m); %Vertical flow on foil due to roll velocity. Will change for every foil strip in all but head/bow seas
        
    end %end strips
    

end %end time          

disp('Motion induced velocity calculation............................done')

%Calculates relative velocities
wrel = wavevel_w - ( HeavevelocityFOIL + VerticalFlowRollMotion - HeaveFOILfrompitchmotion - VerticalFlowPitchDispl);      
urel = U + wavevel_v;

%Theodorsen
W=2*pi/Tp;
W_E = W + (W^2/g)*cos(beta);
KR=W_E*c/(2*U);
C_KR =  1 - (i*KR*A1)/(b1+i*KR) - (i*KR*A2)/(b2+i*KR);
C_KR = abs(C_KR);

%Time series for instantaneous angle of attack 
for t = 1:numT
    for m=1:length(ypos)
        alpha(t,m) = atan(wrel(t,m)/urel(t,m)) - Pdisptot(t);
        alpha(t,m) = alpha(t,m)*C_KR;
    end %end strips
end %end time

disp('AOA calculation................................................done')



%---------------------LIFT-------------------------------------------------
%Instantaneous lift coefficient: Input for Gaussian fit w/7terms below
       a1 =      0.8952;
       b1 =      0.2448;
       c1 =      0.1237;
       a2 =      0.3728;
       b2 =      0.4081;
       c2 =     0.08667;
       a3 =       0.133;
       b3 =      0.4718;
       c3 =     0.03185;
       a4 =      0.2469;
       b4 =      0.1083;
       c4 =      0.0762;
       a5 =           0;
       b5 =      0.7441;
       c5 =    0.003392;
       a6 =      0.5628;%  (-0.0629, 1.189)
       b6 =      0.6323;%  (0.5195, 0.745)
       c6 =      0.2932;%  (0.07691, 0.5094)
       a7 =       0.596;%  (0.2086, 0.9834)
       b7 =       1.044;%  (0.7523, 1.335)
       c7 =      0.3908;%  (0.2193, 0.5623)

       liftcoeff=zeros(numT,length(ypos));
%Time series for lift coefficient, NACA0015 corrected for finite span       
for t=1:numT
    for m=1:length(ypos)
        liftcoeff(t,m)  = a1*exp(-((abs(alpha(t,m))-b1)/c1)^2) + a2*exp(-((abs(alpha(t,m))-b2)/c2)^2) + a3*exp(-((abs(alpha(t,m))-b3)/c3)^2) + a4*exp(-((abs(alpha(t,m))-b4)/c4)^2) + a5*exp(-((abs(alpha(t,m))-b5)/c5)^2) + a6*exp(-((abs(alpha(t,m))-b6)/c6)^2) + a7*exp(-((abs(alpha(t,m))-b7)/c7)^2);
    end %end strips
end %end time

disp('Lift coefficient calculation...................................done')


for t=1:numT %Time series for calculating alphazero, used to find inflow velocity
    for m=1:length(ypos)
        alphazero(t,m) = alpha(t,m)-Pdisptot(t);
    end
end
disp('Alphazero calculation..........................................done')

for t=1:numT %Inflow velocity time series
    for m=1:length(ypos)
        Vin(t,m) = (U+wavevel_v(t,m))/cos(alphazero(t,m));
    end
end
disp('Inflow velocity calculation....................................done')

dL=zeros(numT,length(ypos));
dD=zeros(numT,length(ypos));

%Calculate added mass forces for each time step for each foil strip --> sum

%added mass coefficient:
AC = pi*(c/2)^2*ds*rho;

a_w=zeros(numT,nfreq-1,nstrips);
H_acc = zeros(numT,nfreq-1,nstrips);
R_acc = zeros(numT,nfreq-1,nstrips);
P_acc = zeros(numT,nfreq-1,nstrips);
P_vel = zeros(numT,nfreq-1,nstrips);
            
RelAcc_freq_strip = zeros(numT,nfreq-1,nstrips);
F_added_freq_strip=zeros(numT,nfreq-1,nstrips);

for t=1:numT
    t
    for j=1:nfreq-1
        for m=1:nstrips
            a_w(t,j,m) = k(j)*w(j)*AmpW(j)*exp(k(j)*zc)*cos(w_e(j)*dt*t - k(j)*xc*cos(beta) - k(j)*ypos(m));                                    %Vertical acceleration of fluid at each foil strip
            
            H_acc(t,j,m) = -w_e(j)*AmpW(j)*abs(reRAO_Heave(j))*sin(w_e(j)*dt*t + atan(imRAO_Heave(j)/reRAO_Heave(j)) + epsilon(j));
            R_acc(t,j,m) = -w_e(j)*AmpW(j)*abs(reRAO_Roll(j))*sin(w_e(j)*dt*t + atan(imRAO_Heave(j)/reRAO_Heave(j)) + epsilon(j))*ypos(m);
            P_acc(t,j,m) = -w_e(j)*AmpW(j)*abs(reRAO_Pitch(j))*sin(w_e(j)*dt*t + atan(imRAO_Heave(j)/reRAO_Heave(j)) + epsilon(j))*xc;
            P_vel(t,j,m) = -AmpW(j)*abs(reRAO_Pitch(j))*sin(w_e(j)*dt*t + atan(imRAO_Heave(j)/reRAO_Heave(j)) + epsilon(j))*urel(t,m);
            
            RelAcc_freq_strip(t,j,m) = a_w(t,j,m) - (H_acc(t,j,m) + R_acc(t,j,m) - P_acc(t,j,m) - P_vel(t,j,m));
            F_added_freq_strip(t,j,m)=AC*RelAcc_freq_strip(t,j,m);
        end %end strips
        F_added_freq(t,j)=sum(F_added_freq_strip(t,j,:));
    end %end freq
   F_added(t) = sum(F_added_freq(t,:));
end %end time


for t=1:numT 
    for m=1:length(ypos)
        dL(t,m) = 0.5*rho*Vin(t,m).^2*c*ds*liftcoeff(t,m);
    end
L(t)=sum(dL(t,:));
end


for t=1:numT
    M_lift(t) = L(t)*c/4*cos(alphazero(t,10)-Pdisptot(t)) + F_added(t)*3*c/8;     
end

KF = 10000;

%for each time step
    %find moment. iterate from delta alpha: Mdyn will decrease, whereas
    %Mstif will increase. When delta m = mdyn-mstif = min, you have
    %equillibrium. This value for delta alpha will be the foil pitch at
    %this time instant. Then, calculate lift&drag-->thrust





%for t=1:numT %old version. for reference w/regards thrust
%    for m=1:length(ypos)
%        dL(t,m) = 0.5*rho*Vin(t,m).^2*c*ds*liftcoeff(t,m);
%        dD(t,m) = 0.5*rho*Vin(t,m).^2*c*ds*dragcoeff(t,m); %-->sum
%        dTfromL(t,m) = dL(t,m)*sin(abs(alphazero(t,m)));    %postpone
%        dTfromD(t,m) = - dD(t,m)*cos(abs(alphazero(t,m)));  %postpone
%    end
%    TfromL(t)=sum(dTfromL(t,:));    %postpone. Must come after 
%    TfromD(t)=sum(dTfromD(t,:));    %postpone
%end

%----------------------------DRAG------------------------------------------
%Instantaneous drag coefficient: Input for Gaussian fit w/7 terms below

       a1 =      0.4145;
       b1 =        1.61;
       c1 =      0.1331;
       a2 =      0.8715;
       b2 =      0.8973;
       c2 =      0.3023;
       a3 =           0;
       b3 =        2.87;
       c3 =     0.04941;
       a4 =        1.51;
       b4 =      0.5719;
       c4 =      0.1561;
       a5 =           0;
       b5 =       1.055;
       c5 =    2.22e-14;
       a6 =       1.687;%  (-25.93, 29.31)
       b6 =         1.4;%  (-4.124, 6.924)
       c6 =      0.4044;%  (-21.75, 22.56)
       a7 =      -1.173;%  (-65, 62.65)
       b7 =      0.5746;%  (0.4617, 0.6875)
       c7 =      0.1314;%  (-0.4846, 0.7474)

       dragcoeff=zeros(numT,length(ypos));
       
%Time series for drag coeff NACA       
for t=1:numT
    for m=1:length(ypos)
    dragcoeff(t,m)  = a1*exp(-((abs(alpha(t,m))-b1)/c1)^2) + a2*exp(-((abs(alpha(t,m))-b2)/c2)^2) + a3*exp(-((abs(alpha(t,m))-b3)/c3)^2) + a4*exp(-((abs(alpha(t,m))-b4)/c4)^2) + a5*exp(-((abs(alpha(t,m))-b5)/c5)^2) + a6*exp(-((abs(alpha(t,m))-b6)/c6)^2) + a7*exp(-((abs(alpha(t,m))-b7)/c7)^2); 
    end %end strips
end %end time

disp('Drag coefficient calculation...................................done')






%T = (TfromL+TfromD)*C_KR;

%disp('Lift drag and thrust calculation...............................done')

    %Viscous drag
%Viscdragcoeff=0.015;
%Visc_drag = 0.5*rho*U^2*c*s*Viscdragcoeff;
    
    %Viscous strut resistance
%Visc_strut_res = 0.5*rho*U^2*0.4*4*4*Viscdragcoeff; %b = 0.5 h = 4 Cd=4*foildragcoeff

%T= T - Visc_drag - Visc_strut_res;

%Hs
%Meanthrust = mean(T)
   
        
    %figure                             %Vertical Position of foil
    %plot(timeseries, eta3FOIL)
    %title('Vertical position of foil [m] relative to calm water line, time series')
       
    %figure                             %Wave particle velocity, vertical
    %plot(timeseries, wavevel_w)
    %title('Vertical wave particle velocity at foil position, time series')

    %figure                             %Wave particle velocity, horizontal
    %plot(timeseries, wavevel_v)
    %title('Horizontal wave particle velocity at foil position, time series')     
    
    %figure                             %Foil and ship heave velocity
    %plot(timeseries, HeavevelocityFOIL)
    %title('Heave velocity of ship, time series')
        
    %figure                             %Heave motion of foil from ship pitch motion
    %plot(timeseries, HeaveFOILfrompitchmotion)
    %title('Heave velocity of foil from ship pitch motion, time series')  
        
    %figure                             %Relative vertical fluid motion
    %plot(timeseries,wrel)
    %title('Relative vertical fluid motion, time series') 

    %figure                             %Angle of attack
    %plot(timeseries, alpha(:,10))
    %title('Angle of attack on foil, time series')
        
    %figure
    %plot(timeseries, TfromL);
    %title('Thrust from lift component, time series')
    
    %figure
    %plot(timeseries, TfromD)
    %title('Thrust from drag component, time series')
    