%------------Calculating thrust in irregular waves-------------------------
%-----------------------by foil propulsion---------------------------------
%------------------Written by Eivind Finne Riley---------------------------
%---------------------Part of project thesis-------------------------------
%----------------NTNU, Dept. of Marine Technology--------------------------
%---------------------------December 2014----------------------------------

%--------Number of input frequencies must be 99 w/o modifying script------
%-------------------Input directory must be defined-----------------------
clc
clear all
close all
                                                                                                            
%-------------------------Wave/vessel/ocean/foil data input----------------
Hs     = 3;             %Change    
Tp     = 7;             %Change
beta = 0;                 %Change  (heading) [rad]
rho =1025;                    
g   =9.81;
fp  =1./Tp;
alphajonswap = 0.036 - 0.0056*(Tp/sqrt(Hs));
gamma   = exp(3.484*(1-0.1975*alphajonswap*(Tp^4)/(Hs^2)));
sigma_a = 0.07;
sigma_b = 0.09;
delta_f = 0.1/(2*pi);

U = 6*0.51444;

c = 2;                   %Change
s = 12;                     %Change
aspect = s/c;              
xc=64;                      %Change
zc = -4;                 %Change


yPORT = -s/2;
ySTAR = s/2;

%------Theodorsen coefficients
A1 = 0.165;
A2 = 0.335;
b1 = 0.0455;
b2 = 0.3;

%------The below defines y-positions of foil strips. No. of strips are 20

nstrips=20;

for i=1:nstrips
       if i == 1
           ypos(i) = yPORT + ((ySTAR-yPORT)/(2*nstrips))*i;
       else
           ypos(i) = ypos(i-1) + (ySTAR-yPORT)/nstrips;
       end
end

a = (pi*rho*c*s/nstrips*U^2)/(1+2/aspect);
b = (2*pi*aspect*rho*c*s/nstrips*U^2)/(aspect+2)^2;    



%--------------Calculates the JONSWAP specter------------------------------

w=0.22319:0.013751:pi/2;
f=w./(2*pi);

for j = 1:length(Tp)
    for k  = 1:length (Hs)
        Sf = zeros (1,size(f,2));
        Sw = zeros(1,size(f,2));
        alpha(j,k)=(Hs(1,k)/(4*g))^2*(2*pi*fp(1,j))^4*(0.065*gamma^0.803...
            + 0.135)^-1;
        for i = 1:length(f)
            if f(1,i)<= fp(1,j)
                sigma = sigma_a;
            else 
                sigma = sigma_b;
            end
            Sf(1,i)=alpha(j,k)*g^2*(2*pi)^-4*f(1,i)^-5*exp(-5/4*(fp(1,j...
                )/f(1,i))^4)*gamma^exp(-(f(1,i)/fp(1,j)-1)^2/(2*sigma^2));
            Sw(1,i)=Sf(1,i)/(2*pi);
        end

            
        end
end

%------Imports Real parts of RAOs (MATLAB generated script)-----------------------


filename = 'C:\Matlab\input.re1';

if beta==0

    startRow = 14;                                                              %Changes for each heading. Note that this interval is for numw = 99
    endRow = 705;                                                               %Changes for each heading. Note that this interval is for numw = 99
elseif beta == pi/8
    startRow = 708;
    endRow = 1399;
elseif beta == pi/4
    startRow = 1402;
    endRow = 2093;
elseif beta == 3*pi/8
    startRow=2096;
    endRow=2787;
elseif beta == pi/2
    startRow = 2790;
    endRow = 3481;
elseif beta == 5*pi/8
    startRow = 3484;
    endRow = 4175;
elseif beta == 3*pi/4
    startRow = 4178;
    endRow = 4869;
elseif beta == 7*pi/8
    startRow = 4872;
    endRow = 5563;
elseif beta == pi
    startRow = 5566;
    endRow = 6257;
else
    disp('Invalid value for beta')
    return
end

    
    
    
    formatSpec = '%*3s%16s%[^\n\r]';

fileID = fopen(filename,'r');

dataArray = textscan(fileID, formatSpec, endRow-startRow+1, 'Delimiter', '', 'WhiteSpace', '', 'HeaderLines', startRow-1, 'ReturnOnError', false);

fclose(fileID);

raw = repmat({''},length(dataArray{1}),length(dataArray)-1);
for col=1:length(dataArray)-1
    raw(1:length(dataArray{col}),col) = dataArray{col};
end
numericData = NaN(size(dataArray{1},1),size(dataArray,2));


rawData = dataArray{1};
for row=1:size(rawData, 1);

    regexstr = '(?<prefix>.*?)(?<numbers>([-]*(\d+[\,]*)+[\.]{0,1}\d*[eEdD]{0,1}[-+]*\d*[i]{0,1})|([-]*(\d+[\,]*)*[\.]{1,1}\d+[eEdD]{0,1}[-+]*\d*[i]{0,1}))(?<suffix>.*)';
    try
        result = regexp(rawData{row}, regexstr, 'names');
        numbers = result.numbers;

        invalidThousandsSeparator = false;
        if any(numbers==',');
            thousandsRegExp = '^\d+?(\,\d{3})*\.{0,1}\d*$';
            if isempty(regexp(thousandsRegExp, ',', 'once'));
                numbers = NaN;
                invalidThousandsSeparator = true;
            end
        end
        if ~invalidThousandsSeparator;
            numbers = textscan(strrep(numbers, ',', ''), '%f');
            numericData(row, 1) = numbers{1};
            raw{row, 1} = numbers{1};
        end
    catch me
    end
end

input = cell2mat(raw);
clearvars filename startRow endRow formatSpec fileID dataArray ans raw col numericData rawData row regexstr result numbers invalidThousandsSeparator thousandsRegExp me;

reRAO = input;

for i = 0:98
    reRAO_Heave(i+1)=reRAO(3 + 7*i);
    reRAO_Roll(i+1) = reRAO(4 + 7*i);
    reRAO_Pitch(i+1) = reRAO(5+ 7*i);
end


%--------------------Imports imaginary RAOs--------------------------------


filename = 'C:\Matlab\input.re1';


if beta==0

    startRow = 14;                                                              %Changes for each heading. Note that this interval is for numw = 99
    endRow = 705;                                                               %Changes for each heading. Note that this interval is for numw = 99
elseif beta == pi/8
    startRow = 708;
    endRow = 1399;
elseif beta == pi/4
    startRow = 1402;
    endRow = 2093;
elseif beta == 3*pi/8
    startRow=2096;
    endRow=2787;
elseif beta == pi/2
    startRow = 2790;
    endRow = 3481;
elseif beta == 5*pi/8
    startRow = 3484;
    endRow = 4175;
elseif beta == 3*pi/4
    startRow = 4178;
    endRow = 4869;
elseif beta == 7*pi/8
    startRow = 4872;
    endRow = 5563;
elseif beta == pi
    startRow = 5566;
    endRow = 6257;
else
    disp('Invalid value for beta')
    return
end

formatSpec = '%*19s%s%[^\n\r]';
fileID = fopen(filename,'r');
dataArray = textscan(fileID, formatSpec, endRow-startRow+1, 'Delimiter', '', 'WhiteSpace', '', 'HeaderLines', startRow-1, 'ReturnOnError', false);
fclose(fileID);
raw = repmat({''},length(dataArray{1}),length(dataArray)-1);
for col=1:length(dataArray)-1
    raw(1:length(dataArray{col}),col) = dataArray{col};
end
numericData = NaN(size(dataArray{1},1),size(dataArray,2));

rawData = dataArray{1};
for row=1:size(rawData, 1);
    regexstr = '(?<prefix>.*?)(?<numbers>([-]*(\d+[\,]*)+[\.]{0,1}\d*[eEdD]{0,1}[-+]*\d*[i]{0,1})|([-]*(\d+[\,]*)*[\.]{1,1}\d+[eEdD]{0,1}[-+]*\d*[i]{0,1}))(?<suffix>.*)';
    try
        result = regexp(rawData{row}, regexstr, 'names');
        numbers = result.numbers;
        invalidThousandsSeparator = false;
        if any(numbers==',');
            thousandsRegExp = '^\d+?(\,\d{3})*\.{0,1}\d*$';
            if isempty(regexp(thousandsRegExp, ',', 'once'));
                numbers = NaN;
                invalidThousandsSeparator = true;
            end
        end
        if ~invalidThousandsSeparator;
            numbers = textscan(strrep(numbers, ',', ''), '%f');
            numericData(row, 1) = numbers{1};
            raw{row, 1} = numbers{1};
        end
    catch me
    end
end


R = cellfun(@(x) ~isnumeric(x) && ~islogical(x),raw); 
raw(R) = {NaN}; 

input = cell2mat(raw);

clearvars filename startRow endRow formatSpec fileID dataArray ans raw col numericData rawData row regexstr result numbers invalidThousandsSeparator thousandsRegExp me R;

imRAO = input;

for i = 0:98
    imRAO_Heave(i+1)=imRAO(3 + 7*i);
    imRAO_Roll(i+1) = imRAO(4 + 7*i);
    imRAO_Pitch(i+1) = imRAO(5+ 7*i);
end


%---------Defines the RAOs at midpoint DW values---------------------------
for i=1:98
    imRAO_Heave(i) = (imRAO_Heave(i) + imRAO_Heave(i+1))/2;
    imRAO_Roll(i) = (imRAO_Roll(i) + imRAO_Roll(i+1))/2;
    imRAO_Pitch(i) = (imRAO_Pitch(i) + imRAO_Pitch(i+1))/2;
    reRAO_Heave(i) = (reRAO_Heave(i) + reRAO_Heave(i+1))/2;
    reRAO_Roll(i) = (reRAO_Roll(i) + reRAO_Roll(i+1))/2;
    reRAO_Pitch(i) = (reRAO_Pitch(i) + reRAO_Pitch(i+1))/2;
end

%--------------------------------------------------------------------------


Time = 720;                                         % [s]
dt = 0.5;                                            % [s]
numT = Time/dt;

nfreq = 99; %Should this be 99..? Test later. Ref for j=1:nfreq-1
DW = zeros(nfreq-1,1);


%----------------Begins thrust calculation---------------------------------

%Initiates time series to calculate vertical foil position as function of
%time:

for t=1:numT

timeseries(t) = -dt + t*dt;
    for j=1:nfreq-1
    
        if t==1
         DW(j) = w(j+1)-w(j);                                               %freq interval is slightly different for each freq step due to rounding in shipX
         Amp1W(j)=sqrt(2*Sw(1,j)*DW(j));                                     %Wave elevation amp for each freq. band
         Amp2W(j)=sqrt(2*Sw(1,j+1)*DW(j));
         AmpW(j) = (Amp1W(j)+Amp2W(j))/2;
         w(j) = w(j) + (w(j+1) - w(j))/2;                                   %Sets the frequencies at midpoint DW values
         w_e(j) =w(j) + (w(j)^2)./g.*U*cos(beta);                           %Freq. of encounter
         k(j) = (w(j)^2)/g;                                                 %Wave number
        end
        
    %Determine foil position due to linear wave theory for each freq.
    %component
 
    %AT COG:
        Hdisp(j) = (1/w_e(j))*AmpW(j)*abs(reRAO_Heave(j))*cos(w_e(j)*dt*t + atan(imRAO_Heave(j)/reRAO_Heave(j)));
        Rdisp(j) = (1/w_e(j))*AmpW(j)*abs(reRAO_Roll(j))*cos(w_e(j)*dt*t + atan(imRAO_Roll(j)/reRAO_Roll(j)));
        Pdisp(j) = (1/w_e(j))*AmpW(j)*abs(reRAO_Pitch(j))*cos(w_e(j)*dt*t + atan(imRAO_Pitch(j)/reRAO_Pitch(j)));
    
        YC=0;
 
        
   
    eta3FOILfreq(j) = Hdisp(j) + YC*Rdisp(j) - xc*Pdisp(j);   %Vertical foil position from each frequency
    
    end          %end freq
    HdispSUM(t) = sum(Hdisp); %For viewing only
    PdiskSUM(t) = sum(Pdisp); %For viewing only  
    %%%%Sum up ships's displacements for each time freq. comp. to find ship/foil/strip position at each time instant 
    eta3FOIL(t) = zc + sum(eta3FOILfreq);
    
end  %end timestep

    figure
    plot(timeseries, eta3FOIL)
    title('Vertical position of foil [m] relative to calm water line, time series')

    
%Initiates time series to calculate wave motion at foil position
for t=1:numT
 
    for j = nfreq-1
        wv(j)  = w(j)*AmpW(j)*exp(k(j)*eta3FOIL(t))*cos(w_e(j)*dt*t - k(j)*xc*cos(beta) + k(j)*YC*sin(beta)); 
        uv(j) = -w(j)*AmpW(j)*exp(k(j)*eta3FOIL(t))*sin(w_e(j)*dt*t - k(j)*xc*cos(beta) + k(j)*YC*sin(beta));
    end %End freq
    
    wavevel_w(t) = sum(wv); %Sums vertical wave induced velocities at foil position for all freq. components
    wavevel_v(t) = sum(uv); %Sums horizontal wave induced velocities at foil position for all freq. components

end %End time


    %figure
    %plot(timeseries, wavevel_w)
    %title('Vertical wave particle velocity at foil position, time series')

    %figure
    %plot(timeseries, wavevel_v)
    %title('Horizontal wave particle velocity at foil position, time series')
    
    
for t = 1:numT    
    t
    for j = nfreq-1
        
        %Heave induced motions:
        Hvel(j) = -AmpW(j)*abs(reRAO_Heave(j))*sin(w_e(j)*dt*t + atan(imRAO_Heave(j)/reRAO_Heave(j)));
        
        %Pitch velocity induced motions:
        Pvel(j) = -AmpW(j)*abs(reRAO_Pitch(j))*sin(w_e(j)*dt*t + atan(imRAO_Pitch(j)/reRAO_Pitch(j)));
        foilheavefrompitchmotion(j) = -Pvel(j)*xc;
        
        %Pitch displacement induced motions:
        Pdisp(j) = (1/w_e(j))*AmpW(j)*abs(reRAO_Pitch(j))*cos(w_e(j)*dt*t + atan(imRAO_Pitch(j)/reRAO_Pitch(j)));  
        FoilVerticalFlowFromPitchDisp(j)=U*Pdisp(j);
    end %end freq
        HeaveFOIL(t) = sum(Hvel);
        HeaveFOILfrompitchmotion(t) = sum(foilheavefrompitchmotion);
        VerticalFlowPitchDispl(t) = sum(FoilVerticalFlowFromPitchDisp);
end %end time 
        
    %figure
    %plot(timeseries, HeaveFOIL)
    %title('Heave velocity of ship, time series')      
        
    %figure
    %plot(timeseries, HeaveFOILfrompitchmotion)
    %title('Heave velocity of foil from ship pitch motion, time series')      
                  
wrel = wavevel_w - ( HeaveFOIL - HeaveFOILfrompitchmotion - VerticalFlowPitchDispl);      
urel = U + wavevel_v;

  %figure
  %plot(timeseries,wrel)
  %title('Relative vertical fluid motion, time series') 

for t = 1:numT
    alpha(t) = atan(wrel(t)/urel(t));
end

%figure
%plot(timeseries, alpha)
%title('Angle of attack on foil, time series')
        
      
    
    
    
 %%HUSK: ROLL NOT YET INCLUDED
        
        
        
        
        
        
        
   